@@ -18,9 +18,11 @@
 // See discussion at https://github.com/eclipse-cyclonedds/cyclonedds/issues/830
 
 use cdr::{Bounded, CdrBe, Infinite};
+use rc_box::ArcBox;
 use serde::Deserialize;
 use serde::{de::DeserializeOwned, Serialize};
 use std::io::prelude::*;
+use std::mem::{MaybeUninit};
 use std::ptr::NonNull;
 use std::sync::RwLock;
 use std::{
@@ -65,7 +67,7 @@ pub trait TopicType: Serialize + DeserializeOwned {
 
         let typename =
             std::ffi::CString::new(ty_name_parts).expect("Unable to create CString for type name");
-        println!("Typename:{:?}", &typename);
+        //println!("Typename:{:?}", &typename);
         typename
     }
 
@@ -110,7 +112,7 @@ impl<'a, T> SerType<T> {
     where
         T: DeserializeOwned + Serialize + TopicType,
     {
-        Box::<SerType<T>>::new(SerType {
+         Box::<SerType<T>>::new(SerType {
             sertype: {
                 let mut sertype = std::mem::MaybeUninit::uninit();
                 unsafe {
@@ -124,6 +126,7 @@ impl<'a, T> SerType<T> {
                     );
                     let mut sertype = sertype.assume_init();
                     sertype.set_fixed_size(if T::is_fixed_size() {1} else {0});
+                    sertype.iox_size = std::mem::size_of::<T>() as u32;
                     sertype
                 }
             },
@@ -148,92 +151,121 @@ impl<'a, T> SerType<T> {
     }
 }
 
-/// A sample is simply a smart pointer. The storage for the sample
-/// is in the serdata structure, or allocated by iceoryx in the shared memory
-enum SampleStorage<T> {
+
+
+#[derive(Clone)]
+pub enum SampleStorage<T> {
     Owned(Arc<T>),
-    Loaned(NonNull<T>), 
+    Loaned(Arc<NonNull<T>>),
 }
 
+impl <T>Deref for SampleStorage<T> {
+    type Target = T;
+
+    fn deref(&self) -> &Self::Target {
+        match self {
+            SampleStorage::Owned(t) => t.deref(),
+            SampleStorage::Loaned(t) => unsafe {t.as_ref().as_ref()},
+        }
+    }
+}
+
+impl <T>Drop for SampleStorage<T> {
+    fn drop(&mut self) {
+        match self {
+            SampleStorage::Loaned( t) => {
+                let count = Arc::<NonNull<T>>::strong_count(t); 
+                //println!("Sample Storage reference count : {}", count);
+                
+            },
+            _ => {}
+        }
+    }
+}
+
+
 pub struct Sample<T> {
-    //sample: RwLock<Option<Arc<T>>>,
+    //sample : RwLock<Option<Arc<T>>>,
     sample : RwLock<Option<SampleStorage<T>>>,
 }
 
-impl<T> Sample<T> {
-    pub fn is_loaned(&self) -> bool {
-        let t = self.sample.read().unwrap();
-        match &*t {
-            None => false,
-            Some(t) =>  match t {
-                SampleStorage::Owned(_) => false,
-                SampleStorage::Loaned(_) => true,
-            }  
-        }   
+
+impl<T> Sample<T> 
+where
+        T: TopicType,
+{
+    
+    pub fn get_sample(&self) -> Option<SampleStorage<T>> {
+        if let Ok(t) = self.sample.write() {
+           match t.as_ref() {
+            Some(s) => {
+
+                match s {
+                    SampleStorage::Owned(s) => Some(SampleStorage::Owned(s.clone())),
+                    SampleStorage::Loaned(s) => Some(SampleStorage::Loaned(s.clone())),
+                }
+            }
+            None => None,
+            }
+           
+        } else {
+            None
+        }
     }
 
     pub fn get(&self) -> Option<Arc<T>> {
         let t = self.sample.read().unwrap();
         match &*t {
-            None => None,
-            Some(t) =>  match t {
-                SampleStorage::Owned(t) => Some(t.clone()),
-                SampleStorage::Loaned(t) => None,
-            }  
-        }
-    }
-
-    pub fn get_loaned_mut(&self) -> Option<&T> {
-        let t = self.sample.read().unwrap();
-        match &*t {
-            None => None,
-            Some(t) =>  match t {
-                SampleStorage::Owned(_) => None,
-                SampleStorage::Loaned(t) => unsafe {Some(t.as_ref())},
-            }  
+            Some(SampleStorage::Owned(t)) =>  Some(t.clone()),
+            Some(SampleStorage::Loaned(t)) => 
+            {   println!("Loaned");
+                None},
+            None => {
+                println!("None");
+                None
+            }
         }
     }
 
     pub fn set(&mut self, t: Arc<T>) {
         let mut sample = self.sample.write().unwrap();
-        sample.replace(SampleStorage::<T>::Owned(t));
+        sample.replace(SampleStorage::Owned(t));
     }
 
-    pub unsafe fn set_loaned(&mut self, ptr : *mut T) {
+    pub fn set_loaned(&mut self, t: NonNull<T>) {
         let mut sample = self.sample.write().unwrap();
-        sample.replace(SampleStorage::<T>::Loaned(NonNull::new(ptr).unwrap()));
+        sample.replace(SampleStorage::Loaned(Arc::new(t)));
     }
 
     pub fn clear(&mut self) {
         let mut sample = self.sample.write().unwrap();
-        let _t = sample.take();
+        let t = sample.take();
+
+        match &t {
+        Some(SampleStorage::Owned(o)) => {
+
+        },
+        Some(SampleStorage::Loaned(o)) => {
+            
+        },
+        None => {},
+    }
     }
 
-    pub fn from(it: Arc<T>) -> Self {
-        //Self::Value(it)
+    /* 
+    fn new_loaned(it: NonNull<T>) -> Self {
         Self {
-            sample: RwLock::new(Some(SampleStorage::<T>::Owned(it))),
+            sample :RwLock::new(Some(SampleStorage::Loaned(Arc::new(it)))), 
         }
     }
-}
-
-impl<T> Drop  for Sample<T> {
-    fn drop(&mut self) {
-        if let Ok(mut sample) = self.sample.write() {
-            let sample = sample.take();
-            match sample {
-                Some(s) => match s {
-                    SampleStorage::Owned(s) => {},
-                    SampleStorage::Loaned(s) => {
-                        // Sample data that is allocated in shared memory should not
-                        // be freed here.
-                        
-                    },
-                },
-                None => {},
-            }
-        }   
+    */
+    
+    pub fn from(it: Arc<T>) -> Self {
+        Self {
+            sample: RwLock::new(Some(SampleStorage::Owned(it))),
+        }
     }
+    
 }
 
 impl<T> Default for Sample<T> {
@@ -387,7 +419,7 @@ extern "C" fn free_samples<T>(
     ptrs: *mut *mut std::ffi::c_void,
     len: size_t,
     op: dds_free_op_t,
-) {
+) where T: TopicType {
     let ptrs_v: *mut *mut Sample<T> = ptrs as *mut *mut Sample<T>;
 
     if (op & DDS_FREE_ALL_BIT) != 0 {
@@ -534,7 +566,7 @@ where
         serdata.sample = SampleData::SDKKey;
         
         let mut key_hash_buffer = [0u8;20];
-        let key_hash = &mut key_hash_buffer[4..];
+        let key_hash = &mut key_hash_buffer[4..]; 
 
         for (i, b) in keyhash.iter().enumerate() {
             key_hash[i] = *b;
@@ -554,11 +586,16 @@ unsafe extern "C" fn serdata_from_sample<T>(
     sertype: *const ddsi_sertype,
     kind: u32,
     sample: *const c_void,
-) -> *mut ddsi_serdata {
+) -> *mut ddsi_serdata 
+
+where T : TopicType {
+    println!("Serdata from sample {:?}", sample);
     let mut serdata = SerData::<T>::new(sertype, kind);
     let sample = sample as *const Sample<T>;
     let sample = &*sample;
 
+   
+
     match kind {
         #[allow(non_upper_case_globals)]
         ddsi_serdata_kind_SDK_DATA => {
@@ -634,6 +671,16 @@ where
 unsafe extern "C" fn free_serdata<T>(serdata: *mut ddsi_serdata) {
     // the pointer is really a *mut SerData
     let ptr = serdata as *mut SerData<T>;
+
+    let serdata = &mut * ptr;
+
+    if !serdata.serdata.iox_subscriber.is_null() {
+        let iox_subscriber: *mut iox_sub_t = serdata.serdata.iox_subscriber as *mut iox_sub_t;
+        let chunk = &mut serdata.serdata.iox_chunk;
+        let chunk = chunk as *mut *mut c_void;
+        free_iox_chunk(iox_subscriber, chunk);
+    }
+
     let _data = Box::from_raw(ptr);
     // _data goes out of scope and frees the SerData. Nothing more to do here.
 }
@@ -645,13 +692,21 @@ where
 {
     let serdata = SerData::<T>::mut_ref_from_serdata(serdata);
     match &serdata.sample {
-        SampleData::Uninitialized => panic!("Uninitialized SerData. no size possible"),
+        SampleData::Uninitialized => 0,
         SampleData::SDKKey => serdata.key_hash.key_length() as u32,
         // This function asks for the serialized size so we do this even for SHM Data
-        SampleData::SDKData(sample) | SampleData::SHMData(sample) => {
-            serdata.serialized_size = Some((cdr::calc_serialized_size(&sample.deref())) as u32);
+        SampleData::SDKData(sample) => {
+            serdata.serialized_size = Some((cdr::calc_serialized_size::<T>(&sample.deref())) as u32);
             *serdata.serialized_size.as_ref().unwrap()
         }
+        SampleData::SHMData(_sample) => {
+            // we refuse to serialize SHM data so return 0
+            0
+            /* 
+            serdata.serialized_size = Some((cdr::calc_serialized_size::<T>(sample.as_ref())) as u32);
+            *serdata.serialized_size.as_ref().unwrap()
+            */
+        }
     }
 }
 
@@ -678,6 +733,10 @@ unsafe extern "C" fn serdata_to_ser<T>(
     let buf = buf as *mut u8;
     let buf = buf.add(offset as usize);
 
+    if size == 0 {
+        return
+    }
+
     match &serdata.sample {
         SampleData::Uninitialized => {
             panic!("Attempt to serialize uninitialized serdata")
@@ -690,7 +749,7 @@ unsafe extern "C" fn serdata_to_ser<T>(
             }
         }
         // We may serialize both SDK data as well as SHM Data
-        SampleData::SDKData(serdata) | SampleData::SHMData(serdata) => {
+        SampleData::SDKData(serdata) => {
             let buf_slice = std::slice::from_raw_parts_mut(buf, size as usize);
             if let Err(e) = cdr::serialize_into::<_, T, _, CdrBe>(
                 buf_slice,
@@ -700,6 +759,17 @@ unsafe extern "C" fn serdata_to_ser<T>(
                 panic!("Unable to serialize type {:?} due to {}", T::typename(), e);
             }
         }
+        SampleData::SHMData(serdata) => {
+            let buf_slice = std::slice::from_raw_parts_mut(buf, size as usize);
+            if let Err(e) = cdr::serialize_into::<_, T, _, CdrBe>(
+                buf_slice,
+                serdata.as_ref(),
+                Bounded(size as u64),
+            ) {
+                panic!("Unable to serialize type {:?} due to {}", T::typename(), e);
+            }
+
+        }
     }
 }
 
@@ -729,26 +799,24 @@ where
             iov.iov_base = p as *mut c_void;
             iov.iov_len = len as size_t;
         }
-        SampleData::SDKData(sample) | SampleData::SHMData(sample) => {
+        SampleData::SDKData(sample) => {
             if serdata.cdr.is_none() {
-                if let Some(size) = &serdata.serialized_size {
-                    let mut buffer = Vec::<u8>::with_capacity(*size as usize);
-                    if let Ok(()) = cdr::serialize_into::<_, T, _, CdrBe>(
-                        &mut buffer,
-                        sample.as_ref(),
-                        Infinite,
-                    ) {
-                        serdata.cdr = Some(buffer);
-                    } else {
-                        panic!("Unable to serialize type {:?}", T::typename());
-                    }
-                } else {
-                    if let Ok(data) = cdr::serialize::<T, _, CdrBe>(sample.as_ref(), Infinite) {
-                        serdata.cdr = Some(data);
-                    } else {
-                        panic!("Unable to serialize type {:?}", T::typename());
-                    }
-                }
+                serdata.cdr = serialize_type::<T>(sample, serdata.serialized_size).ok();
+            }
+            if let Some(cdr) = &serdata.cdr {
+                let offset = offset as usize;
+                let last = offset + size as usize;
+                let cdr = &cdr[offset..last];
+                iov.iov_base = cdr.as_ptr() as *mut c_void;
+                iov.iov_len = cdr.len() as size_t;
+            } else {
+                panic!("Unexpected");
+            }
+        }
+
+        SampleData::SHMData(sample) => {
+            if serdata.cdr.is_none() {
+                serdata.cdr = serialize_type::<T>(sample.as_ref(), serdata.serialized_size).ok();
             }
             if let Some(cdr) = &serdata.cdr {
                 let offset = offset as usize;
@@ -764,6 +832,27 @@ where
     ddsi_serdata_addref(&serdata.serdata)
 }
 
+fn serialize_type<T: Serialize>(sample:&T, maybe_size: Option<u32>) -> Result<Vec<u8>,()> {
+    if let Some(size) = maybe_size {
+        let mut buffer = Vec::<u8>::with_capacity(size as usize);
+        if let Ok(()) = cdr::serialize_into::<_, T, _, CdrBe>(
+            &mut buffer,
+            sample,
+            Infinite,
+        ) {
+            Ok(buffer)
+        } else {
+            Err(())
+        }
+    } else {
+        if let Ok(data) = cdr::serialize::<T, _, CdrBe>(sample, Infinite) {
+            Ok(data)
+        } else {
+            Err(())
+        }
+    }
+}
+
 #[allow(dead_code)]
 unsafe extern "C" fn serdata_to_ser_unref<T>(serdata: *mut ddsi_serdata, _iov: *const iovec) {
     let serdata = SerData::<T>::mut_ref_from_serdata(serdata);
@@ -784,7 +873,7 @@ where
     let mut s = Box::<Sample<T>>::from_raw(sample as *mut Sample<T>);
     assert!(!sample.is_null());
 
-    println!("serdata to sample!");
+    //println!("serdata to sample!");
 
     //#[cfg(shm)]
     let ret = if !serdata.serdata.iox_chunk.is_null() {
@@ -797,7 +886,7 @@ where
              if serdata.serdata.kind == ddsi_serdata_kind_SDK_KEY { 
                 compute_key_hash(reader, &mut serdata);
                 serdata.sample = SampleData::SDKKey;
-                false
+                Ok(())
              } else {
                 if let Ok(decoded) = cdr::deserialize::<T>(reader) {
                         if T::has_key() {
@@ -813,38 +902,46 @@ where
                         s.set(sample.clone());
                         serdata.sample = SampleData::SDKData(sample);
                         
-                    false
+                    Ok(())
                 } else {
                     println!("Deserialization error!");
-                    true
+                    Err(())
                 }
             }
         } else {
-            // Not serialized data, we make a sample out of the data
-            println!("Shared memory deser!");
-            assert_eq!((*hdr).data_size as usize, std::mem::size_of::<T>());
+            // Not serialized data, we make a sample out of the data and store it in our sample
+            assert_eq!((*hdr).data_size as usize, std::mem::size_of::<T>());      
             if std::mem::size_of::<T>() == (*hdr).data_size as usize {
                 // Pay Attention here
                 //
                 //
                 let p : *mut T = serdata.serdata.iox_chunk as *mut T;
-                s.set_loaned(p);
-                false
-
+                serdata.sample = SampleData::SHMData(NonNull::new_unchecked(p));
+                Ok(())
             } else {
-                true
+                Err(())
             }      
         }
-    } 
+    } else { Ok(())};    
     
-    else {  // Handle not SHM data normally
-        if let SampleData::SDKData(data) = &serdata.sample {
-            s.set(data.clone());
-            false
-        } else {
-            true
+
+    let ret = if let Ok(()) = ret {
+        match &serdata.sample {
+            SampleData::Uninitialized => true,
+            SampleData::SDKKey => true,
+            SampleData::SDKData(data) => {
+                s.set(data.clone());
+                false
+            },
+            SampleData::SHMData(data) => {
+                s.set_loaned(data.clone());
+                false
+            },
         }
+    } else {
+        true
     };
+ 
     // leak the sample intentionally
     let _intentional_leak = Box::into_raw(s);
     ret
@@ -854,7 +951,7 @@ where
 unsafe extern "C" fn serdata_to_untyped<T>(serdata: *const ddsi_serdata) -> *mut ddsi_serdata {
     let serdata = SerData::<T>::mut_ref_from_serdata(serdata);
 
-    if let SampleData::<T>::SDKData(_d) = &serdata.sample {
+    //if let SampleData::<T>::SDKData(_d) = &serdata.sample {
         let mut untyped_serdata =
             SerData::<T>::new(serdata.serdata.type_, ddsi_serdata_kind_SDK_KEY);
         // untype it
@@ -867,10 +964,10 @@ unsafe extern "C" fn serdata_to_untyped<T>(serdata: *const ddsi_serdata) -> *mut
 
         let ptr = Box::into_raw(untyped_serdata);
         ptr as *mut ddsi_serdata
-    } else {
-        println!("Error: Cannot convert from untyped to untyped");
-        std::ptr::null_mut()
-    }
+    //} else {
+    //    println!("Error: Cannot convert from untyped to untyped");
+    //    std::ptr::null_mut()
+    //}
 }
 
 #[allow(dead_code)]
@@ -880,7 +977,10 @@ unsafe extern "C" fn untyped_to_sample<T>(
     sample: *mut c_void,
     _buf: *mut *mut c_void,
     _buflim: *mut c_void,
-) -> bool {
+) -> bool
+where T: TopicType
+{
+    println!("untyped to sample!");
     if !sample.is_null() {
         let mut sample = Box::<Sample<T>>::from_raw(sample as *mut Sample<T>);
         // hmm. We don't store serialized data in serdata. I'm not really sure how
@@ -925,7 +1025,8 @@ unsafe extern "C" fn print<T>(
     0
 }
 
-fn create_sertype_ops<T>() -> Box<ddsi_sertype_ops> {
+fn create_sertype_ops<T>() -> Box<ddsi_sertype_ops> 
+where T: TopicType {
     Box::new(ddsi_sertype_ops {
         version: Some(ddsi_sertype_v0),
         arg: std::ptr::null_mut(),
@@ -954,28 +1055,34 @@ unsafe extern "C" fn get_sample_size(
 unsafe extern "C" fn from_iox_buffer<T>(
     sertype: *const ddsi_sertype,
     kind: ddsi_serdata_kind,
+    /*_deserialize_hint : bool,*/
     sub: *mut ::std::os::raw::c_void,
     buffer: *mut ::std::os::raw::c_void,
 ) -> *mut ddsi_serdata {
     
-    let mut d = SerData::<T>::new(sertype, kind);
-    let t = SerType::<T>::try_from_sertype(sertype);
-    if t.is_none() {
+    //println!("from_iox_buffer");
+
+    if sertype.is_null() {
         return std::ptr::null::<ddsi_serdata>() as *mut ddsi_serdata;
     }
+
+    let mut d = SerData::<T>::new(sertype, kind);
+    
     // from loaned sample, just take the pointer
     if sub.is_null() {
         d.serdata.iox_chunk = buffer;
-
     } else {
         // from iox buffer
         d.serdata.iox_chunk = buffer;
         d.serdata.iox_subscriber = sub;
-
         let hdr = iceoryx_header_from_chunk(buffer);
         // Copy the key hash (TODO: Check this)
         copy_raw_key_hash(&(*hdr).keyhash.value,&mut d);
     }
+
+    // we don't deserialize right away
+    d.sample = SampleData::SHMData(NonNull::new_unchecked(buffer as *mut T));
+
     let ptr = Box::into_raw(d);
     // only we know this ddsi_serdata is really of type SerData
     ptr as *mut ddsi_serdata
@@ -1027,7 +1134,7 @@ enum SampleData<T> {
     Uninitialized,
     SDKKey,
     SDKData(std::sync::Arc<T>),
-    SHMData(std::sync::Arc<T>),
+    SHMData(NonNull<T>),
 }
 
 impl<T> Default for SampleData<T> {
@@ -1036,6 +1143,12 @@ impl<T> Default for SampleData<T> {
     }
 }
 
+impl <T>Drop for SampleData<T> {
+    fn drop(&mut self) {
+       // Nothing to do here. The SHMData is wrapper with ManuallyDrop so that should take care of it.
+    }
+}
+
 #[derive(PartialEq,Clone)]
 enum KeyHash {
     None,
